# coding=utf-8
"""Tools for deployment of cartodb on a server.

Note this is currently just a place holder. I plan to port these scripts to
fabric scripts:

https://github.com/nighthacker/cartodb-dev

Tim Sutton, August 2013"""
